@@ -45,6 +45,8 @@ sys.stdout.flush()
 #===
 
 def setUpModule():
+    unittest.TestCase.maxDiff = None
+
     datadir = os.getenv("LINK_GRAMMAR_DATA", "")
     if datadir:
         clg.dictionary_set_data_dir(datadir)
@@ -611,7 +613,7 @@ class FSATsolverTestCase(unittest.TestCase):
 class HEnglishLinkageTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls.d, cls.po = Dictionary(), ParseOptions(linkage_limit=300, display_morphology=False)
+        cls.d, cls.po = Dictionary(), ParseOptions(linkage_limit=1000, display_morphology=False)
 
     @classmethod
     def tearDownClass(cls):
@@ -887,6 +889,34 @@ class ZENLangTestCase(unittest.TestCase):
         self.assertEqual(len(linkages), 2)
         self.assertEqual(linkages.next().unused_word_cost(), 1)
 
+    def test_1_step_parsing_with_no_null_links_short(self):
+        self.po = ParseOptions(min_null_count=0, max_null_count=999)
+
+        text = 'This is a test.'
+        sent = Sentence(text, self.d, self.po)
+        self.assertTrue(len(sent.parse()) > 0) # Just check no crashes or leaks
+
+    def test_1_step_parsing_with_no_null_links_long(self):
+        self.po = ParseOptions(min_null_count=0, max_null_count=999)
+
+        text = 12 * 'This is a test. ' # The final blank is essential
+        sent = Sentence(text, self.d, self.po)
+        self.assertTrue(len(sent.parse()) > 0) # Just check no crashes or leaks
+
+    def test_1_step_parsing_with_nulls_short(self):
+        self.po = ParseOptions(min_null_count=0, max_null_count=999, short_length=1)
+
+        text = 'This a'
+        sent = Sentence(text, self.d, self.po)
+        self.assertTrue(len(sent.parse()) > 0) # Just check no crashes or leaks
+
+    def test_1_step_parsing_with_nulls_long(self):
+        self.po = ParseOptions(min_null_count=0, max_null_count=999, short_length=1)
+
+        text = 12 * 'This is a the test ' # The final blank is essential
+        sent = Sentence(text, self.d, self.po)
+        self.assertTrue(len(sent.parse()) > 0) # Just check no crashes or leaks
+
 class JADictionaryLocaleTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -1072,11 +1102,9 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
     linkage diagrams / constituent printings.
     """
     self.__class__.longMessage = True
-    self.maxDiff = None
     if desc != '':
         desc = desc + '-'
     testfile = clg.test_data_srcdir + "parses-" + desc + clg.dictionary_get_lang(lgdict._obj) + ".txt"
-    parses = open(testfile, "rb")
     diagram = None
     constituents = None
     wordpos = None
@@ -1101,13 +1129,15 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
         if opcode != ord('P'):
             self.assertFalse(wordpos, "at {}:{}: Unfinished word-position entry".format(testfile, lineno))
 
+    with open(testfile, 'rb') as _:
+        parses = _.readlines()
+
     for line in parses:
         lineno += 1
         if not is_python2():
             line = line.decode('utf-8')
 
         validate_opcode(ord(line[0])) # Use ord() for python2/3 compatibility
-        prev_opcode = last_opcode
         if line[0] in 'INOCP':
             last_opcode = line[0]
 
@@ -1166,8 +1196,6 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
         else:
             self.fail('\nTest file "{}": Invalid opcode "{}" (ord={})'.format(testfile, line[0], ord(line[0])))
 
-    parses.close()
-
     self.assertIn(last_opcode , 'OCP', "Missing result comparison in " + testfile)
 
 def warning(*msg):
@@ -1206,7 +1234,22 @@ class divert_start(object):
 
     __del__ = divert_end
 
+
 # Decorate Sentence.parse with eqcost_sorted_parse.
 lg_testutils.add_eqcost_linkage_order(Sentence)
 
+# For testing development branches, it may be sometimes useful to use the
+# "test", "debug" and "verbosity" options. The following allows specify them
+# as "tests.py" arguments, interleaved with standard "unittest" arguments.
+
+for i,arg in enumerate(sys.argv):
+    # "-something=" confuses vim's syntax highlight, so add the '=' separately
+    debug = sys.argv.pop(i)[7:] if arg.startswith('-debug' + '=') else ''
+for i,arg in enumerate(sys.argv):
+    test = sys.argv.pop(i)[6:] if arg.startswith('-test' + '=')  else ''
+for i,arg in enumerate(sys.argv):
+    verbosity = int(sys.argv.pop(i)[11:]) if arg.startswith('-verbosity' + '=')  else ''
+if (test or debug or verbosity):
+    ParseOptions = lg_testutils.add_test_option(ParseOptions, test, debug, verbosity)
+
 unittest.main()
