@@ -33,6 +33,8 @@
 #ifndef _API_STRUCTURESH_
 #define _API_STRUCTURESH_
 
+#include <stdint.h>
+
 /* For locale_t */
 #ifdef HAVE_LOCALE_T_IN_LOCALE_H
 #include <locale.h>
@@ -42,10 +44,16 @@
 #endif /* HAVE_LOCALE_T_IN_XLOCALE_H */
 
 #include "api-types.h"
+#include "tracon-set.h"
 #include "corpus/corpus.h"
 #include "memory-pool.h"
 #include "string-set.h"
-#include "string-id.h"
+
+/* Performance tuning.
+ * For short sentences, setting suffix IDs and packing takes more
+ * resources than it saves. If this overhead is improved, these
+ * limit can be set lower. */
+#define SENTENCE_MIN_LENGTH_TRAILING_HASH 6
 
 typedef struct Cost_Model_s Cost_Model;
 struct Cost_Model_s
@@ -117,10 +125,18 @@ struct Sentence_s
 	const char *orig_sentence;  /* Copy of original sentence */
 	size_t length;              /* Number of words */
 	Word  *word;                /* Array of words after tokenization */
-	String_id *connector_suffix_id; /* Used for connector trailing sequence ID */
 	String_set *   string_set;  /* Used for assorted strings */
 	Pool_desc * fm_Match_node;  /* Fast-matcher Match_node memory pool */
 	Pool_desc * Table_connector_pool; /* Count memoizing memory pool */
+	Pool_desc * E_list_pool;
+	Pool_desc * Exp_pool;
+	Pool_desc * X_node_pool;
+	Pool_desc * Disjunct_pool;
+	Pool_desc * Connector_pool;
+
+	/* Connector encoding, packing & sharing. */
+	size_t min_len_encoding;     /* Do it from this sentence length. */
+	void *dc_memblock;           /* For packed disjuncts & connectors. */
 
 	/* Wordgraph stuff. FIXME: create stand-alone struct for these. */
 	Gword *wordgraph;            /* Tokenization wordgraph */
@@ -153,7 +169,6 @@ struct Sentence_s
 #ifdef USE_SAT_SOLVER
 	void *hook;                 /* Hook for the SAT solver */
 #endif /* USE_SAT_SOLVER */
-	void *disjuncts_connectors_memblock;
 };
 
 #endif
