@@ -98,13 +98,7 @@ typedef struct
 } ConTable;
 
 /* On a 64-bit machine, this struct should be exactly 4*8=32 bytes long.
- * Lets try to keep it that way.
- * FIXME: The addition of suffix_id made it > 32 bytes. Fix possibilities:
- * 1. bool multi:1; int suffix_id:15; However, very long sentences may have
- * suffix_id > 32K.
- * 2. Make originating_gword an "int" index.
- * 3. Always pack the connectors, and eliminate the "next" field.
- */
+ * Lets try to keep it that way. */
 struct Connector_struct
 {
 	uint8_t length_limit; /* Can be different than in the descriptor */
@@ -113,10 +107,19 @@ struct Connector_struct
 	                         this could ever connect to.  Computed by
 	                         setup_connectors() */
 	bool multi;           /* TRUE if this is a multi-connector */
-	int suffix_id;        /* Suffix sequence identifier (see preparations.c) */
+	int tracon_id;        /* Tracon identifier (see disjunct-utils.c) */
 	const condesc_t *desc;
 	Connector *next;
-	const gword_set *originating_gword;
+	union
+	{
+		const gword_set *originating_gword; /* Used while and after parsing */
+		/* For pruning use only */
+		struct
+		{
+			int refcount;      /* Memory-sharing reference count */
+			bool shallow;      /* TRUE if this is a shallow connector */
+		};
+	};
 };
 
 void condesc_init(Dictionary, size_t);
@@ -158,7 +161,7 @@ static inline int connector_uc_num(const Connector * c)
 
 
 /* Connector utilities ... */
-Connector * connector_new(const condesc_t *, Parse_Options);
+Connector * connector_new(Pool_desc *, const condesc_t *, Parse_Options);
 void set_connector_length_limit(Connector *, Parse_Options);
 void free_connectors(Connector *);
 
@@ -250,24 +253,13 @@ static inline uint32_t string_hash(const char *s)
 
 /**
  * Hash function for the classic parser linkage memoization.
- * FIXME: Now that it is based on small-integer connector IDs, it may
- * be far from optimal. To be fixed.
  */
 static inline unsigned int pair_hash(unsigned int table_size,
                             int lw, int rw,
-                            const Connector *le, const Connector *re,
-                            unsigned int cost)
+                            int l_id, const int r_id,
+                            unsigned int null_count)
 {
 	unsigned int i;
-	int l_id = 0, r_id = 0;
-
-	if (NULL != le) l_id = le->suffix_id;
-	if (NULL != re) r_id = re->suffix_id;
-
-#ifdef DEBUG
-	assert(((NULL == le) || le->suffix_id) &&
-	       ((NULL == re) || re->suffix_id));
-#endif
 
 #if 0
 	/* hash function. Based on some tests, this seems to be
@@ -283,7 +275,7 @@ static inline unsigned int pair_hash(unsigned int table_size,
 	i += i >> log2_table_size;
 #else
 	/* sdbm-based hash */
-	i = cost;
+	i = null_count;
 	i = lw + (i << 6) + (i << 16) - i;
 	i = rw + (i << 6) + (i << 16) - i;
 	i = l_id + (i << 6) + (i << 16) - i;
@@ -292,4 +284,25 @@ static inline unsigned int pair_hash(unsigned int table_size,
 
 	return i & (table_size-1);
 }
+
+/**
+ * Get the word number of the given tracon.
+ * c is the leading tracon connector. The word number is extracted from
+ * the nearest_word of the deepest connector.
+ * This function depends on setup_connectors() (which initializes
+ * nearest_word). It should not be called after power_prune() (which
+ * changes nearest_word).
+ *
+ * Note: An alternative for getting the word number of a tracon is to keep
+ * it in the tracon list table or in a separate array. Both ways add
+ * noticeable overhead, maybe due to the added CPU cache footprint.
+ * However, if the word number will be needed after power_prune() there
+ * will be a need to keep it in an alternative way.
+ */
+static inline int get_tracon_word_number(Connector *c, int dir)
+{
+	for (; NULL != c->next; c = c->next)
+		;
+	return c->nearest_word + ((dir == 0) ? 1 : -1);
+}
 #endif /* _LINK_GRAMMAR_CONNECTORS_H_ */
