@@ -88,7 +88,7 @@ static void init_table(count_context_t *ctxt, size_t sent_len)
 
 //#define DEBUG_TABLE_STAT
 #if defined(DEBUG) || defined(DEBUG_TABLE_STAT)
-static int hit, miss;  /* Table value found/not found */
+static size_t hit, miss;  /* Table value found/not found */
 #undef DEBUG_TABLE_STAT
 #define DEBUG_TABLE_STAT(x) x
 /**
@@ -102,7 +102,6 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 	int N = 0;          /* NULL table slots */
 	int null_count[256] = { 0 };   /* null_count histogram */
 	int chain_length[64] = { 0 };  /* Chain length histogram */
-	int max_chain_length = 0;
 
 	for (unsigned int i = 0; i < ctxt->table_size; i++)
 	{
@@ -122,14 +121,13 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 		if (c > 0) /* Slot 0 used for length overflow. */
 		{
 			chain_length[c >= (int)ARRAY_SIZE(chain_length) ? 0 : c]++;
-			if (c > max_chain_length) max_chain_length = c;
 			total_c += c;
 		}
 		//if (c != 0) printf("Connector table [%d] c=%d\n", i, c);
 	}
 
 #if __GNUC__
-#define msb(x) (31-__builtin_clz(x))
+#define msb(x) ((int)(CHAR_BIT*sizeof(int)-1)-__builtin_clz(x))
 #else
 #define msb(x) 0
 #endif
@@ -137,15 +135,15 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 	/* The used= value is TotalValues/TableSize (not UsedSlots/TableSize). */
 	printf("Connector table: msb=%d slots=%6d/%6d (%5.2f%%) avg-chain=%4.2f "
 	       "values=%6d (z=%5d nz=%5d N=%5d) used=%5.2f%% "
-	       "acc=%d (hit=%d miss=%d) (sent_len=%zu)\n",
+	       "acc=%zu (hit=%zu miss=%zu) (sent_len=%zu)\n",
 	       msb(ctxt->table_size), used_slots, ctxt->table_size,
 			 100.0f*used_slots/ctxt->table_size, 1.0f*total_c/used_slots,
 	       z+nz, z, nz, N, 100.0f*(z+nz)/ctxt->table_size,
 	       hit+miss, hit, miss, sent->length);
 
 	printf("Chain length histogram:\n");
-	for (int i = 1; i <= max_chain_length; i++)
-		printf("%d: %d\n", i, chain_length[i]);
+	for (size_t i = 1; i < ARRAY_SIZE(chain_length); i++)
+		if (chain_length[i] > 0) printf("%zu: %d\n", i, chain_length[i]);
 	if (chain_length[0] > 0)
 		printf("Chain length > 63: %d\n", chain_length[0]);
 
@@ -173,13 +171,14 @@ static Table_connector * table_store(count_context_t *ctxt,
                                      Connector *le, Connector *re,
                                      unsigned int null_count)
 {
+	int l_id = (NULL != le) ? le->tracon_id : lw;
+	int r_id = (NULL != re) ? re->tracon_id : rw;
+	unsigned int h = pair_hash(ctxt->table_size, lw, rw, l_id, r_id, null_count);
+	Table_connector *t = ctxt->table[h];
 	Table_connector *n = pool_alloc(ctxt->sent->Table_connector_pool);
 
-	unsigned int h = pair_hash(ctxt->table_size, lw, rw, le, re, null_count);
-	Table_connector *t = ctxt->table[h];
-
-	n->l_id = (NULL != le) ? le->suffix_id : lw;
-	n->r_id = (NULL != re) ? re->suffix_id : rw;
+	n->l_id = l_id;
+	n->r_id = r_id;
 	n->null_count = null_count;
 	n->next = t;
 	ctxt->table[h] = n;
@@ -194,10 +193,10 @@ find_table_pointer(count_context_t *ctxt,
                    Connector *le, Connector *re,
                    unsigned int null_count)
 {
-	unsigned int h = pair_hash(ctxt->table_size,lw, rw, le, re, null_count);
+	int l_id = (NULL != le) ? le->tracon_id : lw;
+	int r_id = (NULL != re) ? re->tracon_id : rw;
+	unsigned int h = pair_hash(ctxt->table_size, lw, rw, l_id, r_id, null_count);
 	Table_connector *t = ctxt->table[h];
-	int l_id = (NULL != le) ? le->suffix_id : lw;
-	int r_id = (NULL != re) ? re->suffix_id : rw;
 
 	for (; t != NULL; t = t->next)
 	{
@@ -268,6 +267,22 @@ static Count_bin pseudocount(count_context_t * ctxt,
 	return *count;
 }
 
+/**
+ *  Clamp the given count to INT_MAX.
+ */
+static void parse_count_clamp(Count_bin *total)
+{
+	if (INT_MAX < hist_total(total))
+	{
+		/*  Sigh. Overflows can and do occur, esp for the ANY language. */
+#ifdef PERFORM_COUNT_HISTOGRAMMING
+		total->total = INT_MAX;
+#else
+		*total = INT_MAX;
+#endif /* PERFORM_COUNT_HISTOGRAMMING */
+	}
+}
+
 /**
  * Return the number of optional words strictly between w1 and w2.
  */
@@ -292,7 +307,7 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 	(verbosity_level(D_COUNT_TRACE, "do_count") ? \
 	 prt_error("%-*s", LBLSZ, STRINGIFY(l)) : 0, do_count)
 #define V(c) (!c?"(nil)":connector_string(c))
-#define ID(c,w) (!c?w:c->suffix_id)
+#define ID(c,w) (!c?w:c->tracon_id)
 static Count_bin do_count1(int lineno, count_context_t *ctxt,
                           int lw, int rw,
                           Connector *le, Connector *re,
@@ -377,7 +392,8 @@ static Count_bin do_count(
 	{
 		int nopt_words = num_optional_words(ctxt, lw, rw);
 
-		if ((null_count == 0) || (!ctxt->islands_ok && (lw != -1)) )
+		if ((null_count == 0) ||
+		    (!ctxt->islands_ok && (lw != -1) && (ctxt->sent->word[lw].d != NULL)))
 		{
 			/* The null_count of skipping n words is just n.
 			 * In case the unparsable range contains optional words, we
@@ -411,6 +427,7 @@ static Count_bin do_count(
 		for (int opt = 0; opt <= !!ctxt->sent->word[w].optional; opt++)
 		{
 			null_count += opt;
+
 			for (Disjunct *d = ctxt->sent->word[w].d; d != NULL; d = d->next)
 			{
 				if (d->left == NULL)
@@ -418,9 +435,12 @@ static Count_bin do_count(
 					hist_accumv(&t->count, d->cost,
 						do_count(ctxt, w, rw, d->right, NULL, null_count-1));
 				}
+				parse_count_clamp(&total);
 			}
+
 			hist_accumv(&t->count, 0.0,
 				do_count(ctxt, w, rw, NULL, NULL, null_count-1));
+			parse_count_clamp(&total);
 		}
 		return t->count;
 	}
@@ -631,18 +651,7 @@ static Count_bin do_count(
 					}
 				}
 
-				/* Sigh. Overflows can and do occur, esp for the ANY language. */
-				if (INT_MAX < hist_total(&total))
-				{
-#ifdef PERFORM_COUNT_HISTOGRAMMING
-					total.total = INT_MAX;
-#else
-					total = INT_MAX;
-#endif /* PERFORM_COUNT_HISTOGRAMMING */
-					t->count = total;
-					pop_match_list(mchxt, mlb);
-					return total;
-				}
+				parse_count_clamp(&total);
 			}
 		}
 		pop_match_list(mchxt, mlb);
@@ -651,7 +660,6 @@ static Count_bin do_count(
 	return total;
 }
 
-
 /**
  * Returns the number of ways the sentence can be parsed with the
  * specified null count. Assumes that the fast-matcher and the count
