@@ -10,7 +10,6 @@
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
-
 #include <limits.h>
 #include "api-structures.h"
 #include "count.h"
@@ -208,9 +207,12 @@ static void process_linkages(Sentence sent, extractor_t* pex,
 	 * So just pretend that it's shorter than it is */
 	sent->num_linkages_alloced = sent->num_valid_linkages;
 
-	lgdebug(D_PARSE, "Info: sane_morphism(): %zu of %d linkages had "
-	        "invalid morphology construction\n", N_invalid_morphism,
-	        itry + (itry != maxtries));
+	if (verbosity >= D_USER_INFO)
+	{
+		prt_error("Info: sane_morphism(): %zu of %d linkages had "
+		        "invalid morphology construction\n", N_invalid_morphism,
+		        itry + (itry != maxtries));
+	}
 }
 
 static void sort_linkages(Sentence sent, Parse_Options opts)
@@ -224,88 +226,9 @@ static void sort_linkages(Sentence sent, Parse_Options opts)
 	      sizeof(struct Linkage_s),
 	      (int (*)(const void *, const void *))opts->cost_model.compare_fn);
 
-#ifdef DEBUG
-	/* Skip in case of a timeout - sent->lnkages may be inconsistent then. */
-	if (!resources_exhausted(opts->resources))
-	{
-		/* num_linkages_post_processed sanity check (ONLY). */
-		size_t in;
-		size_t N_linkages_post_processed = 0;
-		for (in=0; in < sent->num_linkages_alloced; in++)
-		{
-			Linkage_info *lifo = &sent->lnkages[in].lifo;
-			if (lifo->discarded) break;
-			N_linkages_post_processed++;
-		}
-		assert(sent->num_linkages_post_processed==N_linkages_post_processed,
-		       "Bad num_linkages_post_processed (%zu!=%zu)",
-		       sent->num_linkages_post_processed, N_linkages_post_processed);
-	}
-#endif
-
 	print_time(opts, "Sorted all linkages");
 }
 
-#define SHORTEST_SENTENCE_TO_PACK 9
-
-/**
- * Pack all disjunct and connectors into one big memory block.
- * This facilitate a better memory caching for long sentences
- * (a performance gain of a few percents).
- *
- * The current Connector struct size is 32 bit, and future ones may be
- * smaller, but still with a power-of-2 size.
- * The idea is to put an integral number of connectors in each cache line
- * (assumed to be >= Connector struct size, e.g. 64 bytes),
- * so one connector will not need 2 cache lines.
- *
- * The allocated memory includes 3 sections , in that order:
- * 1. A block for disjuncts, when it start is not aligned (the disjunct size
- * is currently 56 bytes and cannot be reduced much).
- * 2. A small alignment gap, that ends in a 64-byte boundary.
- * 3. A block of connectors, which is so aligned to 64-byte boundary.
- *
- * FIXME: 1. Find the "best" value for SHORTEST_SENTENCE_TO_PACK.
- * 2. Maybe this check should be done in too stages, the second one
- * will use number of disjunct and connector thresholds.
- */
-static void pack_sentence(Sentence sent)
-{
-	int dcnt = 0;
-	int ccnt = 0;
-
-	if (sent->length < SHORTEST_SENTENCE_TO_PACK) return;
-	for (size_t w = 0; w < sent->length; w++)
-	{
-		Disjunct *d;
-
-		for (d = sent->word[w].d; NULL != d; d = d->next)
-		{
-			dcnt++;
-			for (Connector *c = d->right; c!=NULL; c = c->next) ccnt++;
-			for (Connector *c = d->left; c != NULL; c = c->next) ccnt++;
-		}
-	}
-
-#define CONN_ALIGNMENT sizeof(Connector)
-	size_t dsize = dcnt * sizeof(Disjunct);
-	dsize = ALIGN(dsize, CONN_ALIGNMENT); /* Align connector block. */
-	size_t csize = ccnt * sizeof(Connector);
-	void *memblock = malloc(dsize + csize);
-	Disjunct *dblock = memblock;
-	Connector *cblock = (Connector *)((char *)memblock + dsize);
-	sent->disjuncts_connectors_memblock = memblock;
-
-	for (size_t i = 0; i < sent->length; i++)
-	{
-		Disjunct *word_disjuncts = sent->word[i].d;
-
-		sent->word[i].d = pack_disjuncts_dup(sent->word[i].d, &dblock, &cblock);
-		free_disjuncts(word_disjuncts);
-	}
-}
-
-
 /**
  * classic_parse() -- parse the given sentence.
  * Perform parsing, using the original link-grammar parsing algorithm
@@ -330,68 +253,106 @@ static void pack_sentence(Sentence sent)
  * disjuncts which are not appropriate to continue do_parse() tries with
  * null_count>0. To solve that, we need to restore the original
  * disjuncts of the sentence and call pp_and_power_prune() once again.
+ * Note:
+ * The original sentence disjuncts are saved only when disjunct packing
+ * is done by pack_sentence_for_pruning(), and it is not done for
+ * sentences shorter than a certain limit. In that case the pruning is
+ * done with no null_count==0 optimization so restoring the disjuncts is
+ * no needed.
  */
 void classic_parse(Sentence sent, Parse_Options opts)
 {
 	fast_matcher_t * mchxt = NULL;
 	count_context_t * ctxt = NULL;
-	bool pp_and_power_prune_done = false;
 	Disjunct **disjuncts_copy = NULL;
-	bool is_null_count_0 = (0 == opts->min_null_count);
+	void *saved_memblock = NULL;
 	int max_null_count = MIN((int)sent->length, opts->max_null_count);
+	int min_null_count = opts->min_null_count;
+	bool one_step_parse = (0 == opts->min_null_count) && (0 < max_null_count);
+	int current_prune_level = -1;
+	int needed_prune_level = opts->min_null_count;
+	const int max_prune_level = 1;
 
 	/* Build lists of disjuncts */
 	prepare_to_parse(sent, opts);
 	if (resources_exhausted(opts->resources)) return;
 
-	if (is_null_count_0 && (0 < max_null_count))
+	Tracon_sharing *ts_pruning = pack_sentence_for_pruning(sent);
+	print_time(opts, "Encode connectors for pruning");
+	if (NULL == ts_pruning)
 	{
-		/* Save the disjuncts in case we need to parse with null_count>0. */
-		disjuncts_copy = alloca(sent->length * sizeof(Disjunct *));
-		for (size_t i = 0; i < sent->length; i++)
-			disjuncts_copy[i] = disjuncts_dup(sent->word[i].d);
+		/* The sentence is considered too short for packing. */
+		if (one_step_parse)
+		{
+			/* Since no disjunct packing has been done, we will not be able
+			 * to restore them in case a parsing with null_count>0 is
+			 * needed.  Hence we must not optimize for null_count==0. */
+			needed_prune_level = 1;
+		}
+	}
+	else
+	{
+		free_sentence_disjuncts(sent);
+
+		if (one_step_parse)
+		{
+			/* Save the disjuncts in case we need to parse with null_count>0. */
+			disjuncts_copy = alloca(sent->length * sizeof(Disjunct *));
+			saved_memblock = save_disjuncts(sent, ts_pruning, disjuncts_copy);
+		}
 	}
 
-	for (int nl = opts->min_null_count; nl <= max_null_count; nl++)
+	for (int nl = min_null_count; nl <= max_null_count; nl++)
 	{
 		Count_bin hist;
 		s64 total;
 
-		if (!pp_and_power_prune_done)
+		if ((needed_prune_level <= max_prune_level) &&
+		    (needed_prune_level > current_prune_level))
 		{
-			if (0 != nl)
+			if (one_step_parse &&
+			   (needed_prune_level > 0) && (-1 != current_prune_level))
 			{
-				pp_and_power_prune_done = true;
-				if (is_null_count_0)
-				{
-					opts->min_null_count = 1; /* Don't optimize for null_count==0. */
+				/* We need to prune *again*. Restore the original disjuncts. */
+				restore_disjuncts(sent, disjuncts_copy, saved_memblock, ts_pruning);
+				/* Free the memory of the previous pack_sentence_for_parsing(). */
+				free(sent->dc_memblock);
+				sent->dc_memblock = NULL;
+			}
 
-					/* We are parsing now with null_count>0, when previously we
-					 * parsed with null_count==0. Restore the save disjuncts. */
-					free_sentence_disjuncts(sent);
-					for (size_t i = 0; i < sent->length; i++)
-						sent->word[i].d = disjuncts_copy[i];
-					disjuncts_copy = NULL;
+			current_prune_level = needed_prune_level;
+			opts->min_null_count = needed_prune_level;
+
+			pp_and_power_prune(sent, ts_pruning, opts);
+			if (one_step_parse)
+				needed_prune_level++; /* In case we need to prune again. */
+
+			Tracon_sharing *ts_parsing = pack_sentence_for_parsing(sent);
+			print_time(opts, "Encode connectors for parsing");
+			if (NULL != ts_parsing)
+			{
+				sent->dc_memblock = ts_parsing->memblock;
+				free_tracon_sharing(ts_parsing);
+			}
+
+			if (!one_step_parse || (max_prune_level == opts->min_null_count))
+			{
+				/* At this point no further pruning will be done. Free the
+				 * pruning tracon stuff here instead of at the end. */
+				if (NULL != ts_pruning)
+				{
+					free(ts_pruning->memblock);
+					free_tracon_sharing(ts_pruning);
+					ts_pruning = NULL;
+					free(saved_memblock);
 				}
 			}
-			pp_and_power_prune(sent, opts);
-			pack_sentence(sent);
-			bool real_suffix_ids = set_connector_hash(sent);
-			if (is_null_count_0) opts->min_null_count = 0;
+
+			gword_record_in_connector(sent);
 			if (resources_exhausted(opts->resources)) break;
 
-			/* If parsing with no nulls or we are using fake suffix_id's
-			 * (because it is a short sentence) always allocate the count
-			 * connector table. Else allocate it only if this is a parse with
-			 * nulls only. So in case of one-step parse (min_null_count==0 &&
-			 * max_null_count>0) the table is shared and the null_count==0
-			 * parsing will be inferred from the table. */
-			if (!real_suffix_ids || (nl == 0) || !is_null_count_0)
-			{
-				free_count_context(ctxt, sent);
-				ctxt = alloc_count_context(sent);
-			}
-
+			free_count_context(ctxt, sent);
+			ctxt = alloc_count_context(sent);
 			free_fast_matcher(sent, mchxt);
 			mchxt = alloc_fast_matcher(sent);
 			print_time(opts, "Initialized fast matcher");
@@ -404,9 +365,6 @@ void classic_parse(Sentence sent, Parse_Options opts)
 		hist = do_parse(sent, mchxt, ctxt, sent->null_count, opts);
 		total = hist_total(&hist);
 
-		lgdebug(D_PARSE, "Info: Total count with %zu null links:   %lld\n",
-		        sent->null_count, total);
-
 		/* total is 64-bit, num_linkages_found is 32-bit. Clamp */
 		total = (total > INT_MAX) ? INT_MAX : total;
 		total = (total < 0) ? INT_MAX : total;
@@ -414,6 +372,12 @@ void classic_parse(Sentence sent, Parse_Options opts)
 		sent->num_linkages_found = (int) total;
 		print_time(opts, "Counted parses");
 
+		if (verbosity >= D_USER_INFO)
+		{
+			prt_error("Info: Total count with %zu null links: %lld\n",
+			        sent->null_count, total);
+		}
+
 		extractor_t * pex = extractor_new(sent->length, sent->rand_state);
 		bool ovfl = setup_linkages(sent, pex, mchxt, ctxt, opts);
 		process_linkages(sent, pex, ovfl, opts);
@@ -422,17 +386,25 @@ void classic_parse(Sentence sent, Parse_Options opts)
 		post_process_lkgs(sent, opts);
 
 		if (sent->num_valid_linkages > 0) break;
+		if (verbosity >= D_USER_INFO)
+		{
+			if (sent->num_linkages_post_processed > 0)
+				prt_error("Info: All linkages had P.P. violations.\n"
+				        "Consider to increase the linkage limit.\n"
+				        "At the command line, use !limit\n");
+		}
+
 		if ((0 == nl) && (0 < max_null_count) && verbosity > 0)
 			prt_error("No complete linkages found.\n");
-
-		//if (sent->num_linkages_found > 0 && nl>0) printf("NUM_LINKAGES_FOUND %d\n", sent->num_linkages_found);
 	}
+	opts->min_null_count = min_null_count;
 	sort_linkages(sent, opts);
 
-	if (NULL != disjuncts_copy)
+	if (NULL != ts_pruning)
 	{
-		for (size_t i = 0; i < sent->length; i++)
-			free_disjuncts(disjuncts_copy[i]);
+		free(ts_pruning->memblock);
+		free_tracon_sharing(ts_pruning);
+		free(saved_memblock);
 	}
 	free_count_context(ctxt, sent);
 	free_fast_matcher(sent, mchxt);
